/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition;

import com.vii.brillien.ignition.classloading.PresenceClassLoader;
import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.services.IOServices;
import com.vii.streamline.services.json.JsonServices;
import com.vii.streamline.services.reflection.ReflectionServices;
import org.slf4j.impl.BrillienLoggerAdapter;

import java.io.File;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import static com.vii.brillien.ignition.BrillienContext.basicLog;
import static com.vii.brillien.ignition.BrillienContext.initServices;

/**
 * Class containing life-cycle management methods for the Brillien
 */
public class LogiqueSceptre {

    private static Class brillienServicesClass;

    public static Class getBrillienServicesClass() {
        return brillienServicesClass;
    }

    /**
     * Starts the Brillien application server
     * @throws com.vii.brillien.kernel.BrillienException
     */
    public static void startLiaison() throws Exception {
        startLiaison( null );     
    }

    /**
     * Starts the Brillien application server
     * @throws com.vii.brillien.kernel.BrillienException
     */
    public static void startLiaison(Logger logger) throws Exception {

        if( !new File( BrillienStatics.CONFIGURATION_DIR + "config.json" ).exists() ){
            basicLog( Level.SEVERE, "Config file is missing: " + BrillienStatics.CONFIGURATION_DIR + "config.json" );
            throw new Exception( "Config file is missing: " + BrillienStatics.CONFIGURATION_DIR + "config.json" );
        }

        basicLog( Level.CONFIG, "Loading config file: " + BrillienStatics.CONFIGURATION_DIR + "config.json" );

        BrillienLoggerAdapter.mainLogger = logger;

        PresenceClassLoader primordialClassLoader = new PresenceClassLoader(
                ClassLoader.getSystemClassLoader()
        );
        primordialClassLoader.appendURL( new File( BrillienStatics.DEP_LIB_DIR ) );
        primordialClassLoader.appendURL( new File( BrillienStatics.LIB_DIR ) );

        initServices(
                primordialClassLoader,
                (Map<String, Object>)JsonServices.parseJSON( IOServices.getStreamAsString( IOServices.getInputStream( BrillienStatics.CONFIGURATION_DIR + "config.json" ) ) )
        );
        /*
        if( BrillienContext.PACKET_REPLY_TIMEOUT > 0 )
            SmackConfiguration.setPacketReplyTimeout( BrillienContext.PACKET_REPLY_TIMEOUT );
        */
        //        null,
        //        BrillienStatics.DEP_LIB_DIR, BrillienStatics.LIB_DIR
        //); //primordialClassLoader.setDescenter( true );


        brillienServicesClass = primordialClassLoader.loadClass("com.vii.brillien.core.management.BrillienServices");
                
        ReflectionServices.invokeMethod(
                ReflectionServices.getMethod(brillienServicesClass, "initServices" ),
                brillienServicesClass
        );

        ReflectionServices.invokeMethod(
                ReflectionServices.getMethod(brillienServicesClass, "startServices" ),
                brillienServicesClass
        );
        
    }

    /**
     * Stops the Brillien application server
     * @throws com.vii.brillien.kernel.BrillienException
     */
    public static void stopLiaison() throws BrillienException {
        try {
            ReflectionServices.invokeMethod(
                ReflectionServices.getMethod(brillienServicesClass, "stopServices"),
                    brillienServicesClass
            );
        } catch (Exception e) {
            e.printStackTrace();  
        }

        //ApplicationServices.stopServices();
    }

    /**
     * Restarts the Brillien application server
     * @throws com.vii.brillien.kernel.BrillienException
     */
    public static void restartLiaison() throws Exception {
        stopLiaison();
        startLiaison();
    }


    public static void main(String[] args) throws Exception {
        startLiaison();
    }
}

