package com.vii.brillien.ignition.jmx.mbeans;

import javax.management.Attribute;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

/**
 * Abstract class for every MXBean proxy type
 * Created with IntelliJ IDEA.
 * User: vii
 * Date: 2012.04.28.
 * Time: 17:25
 * To change this template use File | Settings | File Templates.
 */
public class AbstractBeanWrapper {

    /**
     * MBeanServerConnection instance 
     */
    protected MBeanServerConnection     mbsc;
    /**
     * Name of the MXBean
     */
    protected ObjectName                name;

    public AbstractBeanWrapper(MBeanServerConnection mbsc, ObjectName name) {
        this.mbsc = mbsc;
        this.name = name;
    }

    /**
     * Invokes a service on the MXBean instance
     */
    protected <T> T invoke( String operationName, T defaultValue ){
        return invoke( operationName, null, defaultValue );
    }

    /**
     * Invokes a service on the MXBean instance
     */
    protected <T> T invoke( String operationName, Object[] params, T defaultValue ){
        return invoke( operationName, params, null, defaultValue );
    }

    /**
     * Invokes a service on the MXBean instance
     */
    protected <T> T invoke( String operationName, Object[] params, String[] signature, T defaultValue ){
        try {
            return (T)mbsc.invoke( name, operationName, params, signature );
        } catch (Exception e) {
            e.printStackTrace();
        }
        return defaultValue;
    }

    /**
     * Gets the value of a property published by the MXBean instance
     */
    protected <T> T get( String parameterName, T defaultValue ){
        try {
            return (T)mbsc.getAttribute( name, parameterName );
        } catch (Exception e) {
            e.printStackTrace();
        }
        return defaultValue;
    }

    /**
     * Sets the value of a property published by the MXBean instance
     */
    protected void set( Attribute a ){
        try {
            mbsc.setAttribute( name, a );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }



}
 