/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.jmx.mbeans;

import com.vii.brillien.ignition.jmx.BeanStatics;
import com.vii.streamline.structures.collections.InnerList;

import javax.management.MBeanServerConnection;
import javax.management.openmbean.TabularDataSupport;
import java.lang.management.RuntimeMXBean;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Specialized AbstractBeanWrapper to provide RuntimeMXBean services
 */
public class RuntimeImpl extends AbstractBeanWrapper implements RuntimeMXBean {

    public RuntimeImpl(MBeanServerConnection mbsc) {
        super(mbsc, BeanStatics.RUNTIME_BEAN_NAME);
    }

    public String getName() {
        return get(  "Name", null );
    }

    public String getVmName() {
        return get(  "VmName", null );
    }

    public String getVmVendor() {
        return get(  "VmVendor", null );
    }

    public String getVmVersion() {
        return get(  "VmVersion", null );
    }

    public String getSpecName() {
        return get(  "SpecName", null );
    }

    public String getSpecVendor() {
        return get(  "SpecVendor", null );
    }

    public String getSpecVersion() {
        return get(  "SpecVersion", null );
    }

    public String getManagementSpecVersion() {
        return get(  "ManagementSpecVersion", null );
    }

    public String getClassPath() {
        return get(  "ClassPath", null );
    }

    public String getLibraryPath() {
        return get(  "LibraryPath", null );
    }

    public boolean isBootClassPathSupported() {
        return get(  "BootClassPathSupported", false );
    }

    public String getBootClassPath() {
        return get(  "BootClassPath", null );
    }

    public List<String> getInputArguments() {
        return new InnerList<String>( (String[])get(  "InputArguments", null ) );
    }

    /*public String[] getInputArguments() {
        return get(  "InputArguments", null );
    }*/

    public long getUptime() {
        return get(  "Uptime", -1l );
    }

    public long getStartTime() {
        return get(  "StartTime", -1l );
    }

    public Map<String, String> getSystemProperties() {
        TabularDataSupport tb = get(  "SystemProperties", null );
        Set<Map.Entry<Object,Object>> entry = tb.entrySet();

        HashMap<String, String> res = new HashMap<String, String>();
        for( Map.Entry e : entry )
            res.put( e.getKey().toString(), e.getValue().toString() );

        return res;
    }
}
