/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport.jsonrpc;

import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.ignition.transport.TransportStatics;
import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.structures.collections.InnerList;
import com.vii.streamline.structures.collections.InnerMap;
import com.vii.streamline.web.HttpServices;

import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.vii.streamline.jsonrpc.exchange.ExchangeStatics.*;

/**
 * Listener class polling the url stored in the member of {@JsonRpcMediator}.
 */
public class JsonRpcPoller implements Runnable {

    /**
     * AmqpMediator instance
     */
    private JsonRpcMediator         mediator;

    /**
     * Name of the queue to be listened.
     */
    private String                  queuename;

    /**
     * Flag flield to set the listening status of the Listener
     */
    private Boolean                 listen;


    private InnerMap<String, Class> typeMap;

    {
        listen  = true;

        typeMap = new InnerMap<String, Class>(
                        ".result", List.class, ".result[]", JsonRpcCommunication.class
                  );
    }

    public JsonRpcPoller(JsonRpcMediator mediator, String queuename) {
        this.mediator = mediator;
        this.queuename = queuename;
    }

    /**
     * Get method for field mediator
     */
    public JsonRpcMediator getMediator() {
        return mediator;
    }

    /**
     * Set method for field mediator
     */
    public void setMediator(JsonRpcMediator mediator) {
        this.mediator = mediator;
    }

    /**
     * Get method for field queuename
     */
    public String getQueuename() {
        return queuename;
    }

    /**
     * Set method for field queuename
     */
    public void setQueuename(String queuename) {
        this.queuename = queuename;
    }

    /**
     * Get method for field listen
     */
    public Boolean getListen() {
        return listen;
    }

    /**
     * Set method for field listen
     */
    public void setListen(Boolean listen) {
        this.listen = listen;
    }

    @Override
    public void run() {
        if(listen){
            try{
                List<JsonRpcCommunication> resps = HttpServices.<List>sendJsonRpcMessage(
                        mediator.getUrl(),
                        JSON_RPC_METHOD_PREFIX + JSON_RPC_METHOD_SEPARATOR + JSON_RPC_METHOD_POLL_POSTFIX,
                        new InnerList( queuename ),
                        typeMap
                );
                for( JsonRpcCommunication comm : resps )
                    mediator.receiveCommunication( comm );
            } catch (Exception e) {
                BrillienContext.exceptionLog(this.getClass().getName(), "run", e);
            }

            if( listen )
                ThreadServices.getStpe().schedule(
                        this, TransportStatics.POLLING_INTERVAL, TimeUnit.MILLISECONDS
                );
        }
    }

}
