/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport.mq;

import com.vii.brillien.ignition.transport.BrillienCommunication;
import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.services.error.StreamLineException;

import java.util.concurrent.ConcurrentHashMap;

/**
 * Utility class implementing a Queue-based message management used by MqMediator instances.
 */
public class MqStation {
    
    private static ConcurrentHashMap<String, MqMediator>        mediators;

    static{
        mediators       = new ConcurrentHashMap<String, MqMediator>();
    }

    /**
     * Registers a Mediator to the given queue
     */
    public static void registerMediator(String entityName, MqMediator mediator) throws BrillienException {
        if( mediators.containsKey( entityName.toLowerCase() ) )
            throw new BrillienException( StreamLineException.ERROR_INTERNAL_ERROR, "Mediator entity name already known!", entityName );

        mediators.put( entityName.toLowerCase(), mediator );
    }

    /**
     * Unregisters the mediator associated to the given queue
     */
    public static void unregisterMediator(String entityName){
        mediators.remove( entityName.toLowerCase() );
    }

    /**
     * Publishes a new communication invoked by the registered MqMediator instances.
     */
    public static void publish(final BrillienCommunication comm) throws BrillienException {
        if( !mediators.containsKey( comm.getRecipient().toLowerCase() ) )
            throw new BrillienException(StreamLineException.ERROR_INVALID_REQUEST, "Recipient is not reckognised! " + comm );

        try {
            mediators.get( comm.getRecipient().toLowerCase() ).receiveCommunication( comm );
        } catch (BrillienException e) {
            e.printStackTrace();
        }
    }
    
}
